@@ -25,6 +25,7 @@
 #include <libintl.h>
 #include <hildon/hildon.h>
 #include <rtcom-eventlogger/eventlogger.h>
+#include <rtcom-eventlogger-plugins/chat.h>
 #include <sqlite3.h>
 #include <string.h>
 #include <gconf/gconf-client.h>
@@ -85,8 +86,8 @@
 static const gchar *conv_services[] = {"RTCOM_EL_SERVICE_SMS",
                                        "RTCOM_EL_SERVICE_CHAT",
                                        NULL};
-static const gchar *conv_event_types[] = {"RTCOM_EL_EVENTTYPE_SMS_INBOUND",
-                                          "RTCOM_EL_EVENTTYPE_CHAT_INBOUND",
+static const gchar *conv_event_types[] = {"RTCOM_EL_EVENTTYPE_SMS_MESSAGE",
+                                          "RTCOM_EL_EVENTTYPE_CHAT_MESSAGE",
                                           NULL};
 
 typedef enum {
@@ -142,7 +143,8 @@ struct _ELHomeAppletPrivate
         gchar *contact_id;
         gchar *remote_id;
         gchar *local_id;
-        gchar *group_uid;
+        gchar *channel;
+        int flags;
         OssoABookContact *contact;
 
         gboolean time_fmt_24h;
@@ -442,7 +444,7 @@ expose_event (GtkWidget *self, GdkEventExpose *event)
         if (!priv->message) {
                 rounded_rectangle (cr,
                                    0, HEADER_HEIGHT,
-                                   C_WIDTH, C_HEIGHT,
+                                   C_WIDTH, C_HEIGHT - HEADER_HEIGHT,
                                    BOX_RADIOUS,
                                    ROUND_CORNER_BL | ROUND_CORNER_BR);
                 cairo_close_path (cr);
@@ -822,9 +824,9 @@ clean_state (ELHomeApplet *self)
                 g_free (priv->remote_id);
                 priv->remote_id = NULL;
         }
-        if (priv->group_uid) {
-                g_free (priv->group_uid);
-                priv->group_uid = NULL;
+        if (priv->channel) {
+                g_free (priv->channel);
+                priv->channel = NULL;
         }
 }
 
@@ -976,62 +978,44 @@ static void
 show_event (ELHomeApplet *self, RTComElIter *it)
 {
         ELHomeAppletPrivate *priv = self->priv;
-        const gchar *remote = NULL;
+        gchar *remote = NULL;
         gchar *received = NULL;
-        GValueArray *event = NULL;
 
         if (it && rtcom_el_iter_first (it)) {
+                time_t received_t;
 
-                event = rtcom_el_iter_get_valuearray (it,
-                                                      "id",
-                                                      "start-time",
-                                                      "local-uid",
-                                                      "remote-uid",
-                                                      "remote-name",
-                                                      "remote-ebook-uid",
-                                                      "free-text",
-                                                      "group-uid",
-                                                      NULL);
-                if (event) {
-                        time_t received_t;
-#define _VARR_DUP_STR(array, i) g_value_dup_string (g_value_array_get_nth ((array), (i)))
-
-                        priv->event_id = g_value_get_int (g_value_array_get_nth (event, 0));
-                        received_t = g_value_get_int (g_value_array_get_nth (event, 1));
+                if (rtcom_el_iter_get_values (it,
+                                              "id", &priv->event_id,
+                                              "start-time", &received_t,
+                                              "local-uid", &priv->local_id,
+                                              "remote-uid", &priv->remote_id,
+                                              "remote-name", &remote,
+                                              "remote-ebook-uid", &priv->contact_id,
+                                              "free-text", &priv->message,
+                                              "channel", &priv->channel,
+                                              "flags", &priv->flags,
+                                              NULL)) {
                         received = format_time (received_t, priv->time_fmt_24h);
-                        priv->local_id = _VARR_DUP_STR (event, 2);
-                        priv->remote_id = _VARR_DUP_STR (event, 3);
-                        if (priv->remote_id && priv->remote_id[0]) {
-                                remote = g_value_get_string (g_value_array_get_nth (event, 4));
-                                if (!remote)
-                                        remote = priv->remote_id;
-                                priv->contact_id = _VARR_DUP_STR (event, 5);
-                        }
-                        else if (priv->remote_id) {
+
+                        if (priv->remote_id && !priv->remote_id[0]) {
                                 g_free (priv->remote_id);
                                 priv->remote_id = NULL;
                         }
-
-                        priv->message = _VARR_DUP_STR (event, 6);
-                        priv->group_uid = _VARR_DUP_STR (event, 7);
-
-#undef _VARR_DUP_STR
                 }
+                else
+                        priv->event_id = -1;
         }
-        else {
+        else
                 priv->event_id = -1;
-        }
 
-        if (priv->message) {
+        if (priv->message)
                 gtk_widget_hide (priv->empty);
-        }
-        else {
+        else
                 gtk_widget_show (priv->empty);
-        }
 
         gtk_label_set_text (GTK_LABEL (priv->received), received);
 
-        if (remote)
+        if (remote && remote[0])
                 gtk_label_set_text (GTK_LABEL (priv->sender), remote);
         else
                 gtk_label_set_text (GTK_LABEL (priv->sender), priv->remote_id);
@@ -1043,11 +1027,10 @@ show_event (ELHomeApplet *self, RTComElIter *it)
                 priv->message_surface = NULL;
         }
 
-        if (event)
-                g_value_array_free (event);
-
         gtk_widget_hide (priv->cut_message);
         gtk_widget_queue_draw (GTK_WIDGET (self));
+
+        g_free (remote);
 }
 
 static RTComElIter*
@@ -1066,6 +1049,7 @@ make_query (RTComEl *el, gint event_id)
         else {
                 rtcom_el_query_prepare (query,
                                         "is-read", FALSE, RTCOM_EL_OP_EQUAL,
+                                        "outgoing", FALSE, RTCOM_EL_OP_EQUAL,
                                         "service", conv_services, RTCOM_EL_OP_IN_STRV,
                                         "event-type", conv_event_types, RTCOM_EL_OP_IN_STRV,
                                         NULL);
@@ -1413,9 +1397,8 @@ open_conversation (ELHomeApplet *self)
 {
         ELHomeAppletPrivate *priv = self->priv;
         McAccount *account;
-        const gchar *persistent_id = NULL;
 
-        if (!((priv->remote_id || priv->group_uid) && priv->local_id))
+        if (!((priv->remote_id || priv->channel) && priv->local_id))
                 return;
 
         account = osso_abook_account_manager_lookup_by_name (NULL,
@@ -1423,21 +1406,18 @@ open_conversation (ELHomeApplet *self)
         if (!account)
                 return;
 
-        if (priv->group_uid &&
-            g_str_has_prefix (priv->group_uid, "group:")) {
-                persistent_id = strchr (priv->group_uid, '-');
-                if (persistent_id)
-                        persistent_id++;
-        }
+        if ((priv->flags & RTCOM_EL_FLAG_CHAT_GROUP) &&
+            (priv->flags & RTCOM_EL_FLAG_CHAT_OPAQUE)) {
+                if (!priv->channel)
+                        return;
 
-        if (persistent_id && persistent_id[0] != '\0') {
                 GHashTable *properties = tp_asv_new
                         (TP_IFACE_CHANNEL ".ChannelType", G_TYPE_STRING,
                          TP_IFACE_CHANNEL_TYPE_TEXT,
                          TP_IFACE_CHANNEL ".TargetHandleType", G_TYPE_UINT,
                          TP_HANDLE_TYPE_NONE,
                          RTCOM_TP_IFACE_CHANNEL_INTERFACE_PERSISTENT ".PersistentID",
-                         G_TYPE_STRING, persistent_id,
+                         G_TYPE_STRING, priv->channel,
                          NULL);
 
                 mc_account_channelrequest_ht (account,
@@ -1449,13 +1429,23 @@ open_conversation (ELHomeApplet *self)
 
                 g_hash_table_unref (properties);
         }
-        else if (priv->remote_id) {
+        else {
                 McAccountChannelrequestData request;
 
                 MC_ACCOUNT_CRD_INIT (&request);
                 MC_ACCOUNT_CRD_SET (&request, channel_type, TP_IFACE_QUARK_CHANNEL_TYPE_TEXT);
-                MC_ACCOUNT_CRD_SET (&request, target_handle_type, TP_HANDLE_TYPE_CONTACT);
-                MC_ACCOUNT_CRD_SET (&request, target_id, priv->remote_id);
+
+                if ((priv->flags & RTCOM_EL_FLAG_CHAT_ROOM) &&
+                    priv->channel) {
+                        MC_ACCOUNT_CRD_SET (&request, target_handle_type, TP_HANDLE_TYPE_ROOM);
+                        MC_ACCOUNT_CRD_SET (&request, target_id, priv->channel);
+                }
+                else if (priv->remote_id) {
+                        MC_ACCOUNT_CRD_SET (&request, target_handle_type, TP_HANDLE_TYPE_CONTACT);
+                        MC_ACCOUNT_CRD_SET (&request, target_id, priv->remote_id);
+                }
+                else
+                        return;
 
                 mc_account_channelrequest (account,
                                            &request,
